*------------------------------------------------------------------------------------------------------------*
*                          营商环境的决定因素：基于政务服务能力的视角   
*                                      使用stata SE 16版本运行
*------------------------------------------------------------------------------------------------------------*



clear


*********
********* 设置工作路径
*********
// cd D:\桌面\数据



capture log close
log using "日志.log", replace 



global controls  " control_z1-control_z6 control_x1-control_x4 "  // 控制变量
/*
个体控制变量：control_z1-control_z6。具体含义如下
control_z1:受访企业办事人员性别
control_z2:受访企业办事人员年龄
control_z3:受访办事人员所在企业的行业
control_z4:受访办事人员所在企业的员工规模
control_z5:受访办事人员所在企业的所有制
control_z6:受访办事人员所在企业的成立年限

地市控制变量：control_x1-control_x4。具体含义如下
control_x1:人均GDP
control_x2:财政压力
control_x3:城镇化率
control_x4:官员更替
*/






*------------------------------------------------------------------------------------------------------------*
*                                           正文图表
*                                按正文出现顺序：表1、图4、表2、表3
*------------------------------------------------------------------------------------------------------------*

*****************************************
**** 表1：基准结果
*****************************************	

use  "data.dta", clear


* 第1-3列：被解释变量=是否跑一次
reghdfe p1 b  $controls   ,vce(cluster prefecture_id) absorb( prefecture_id  province_id#year)   
outreg2 using "Results\表1.doc",  keep(b ) se  bdec(3) rdec(3) nocons replace

probit  p1 b  $controls  i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
margins, dydx(b)  //平均边际效应
outreg2 using  "Results\表1.doc",  keep(b ) se e(r2_p) bdec(3) rdec(3) nocons append

logit  p1 b  $controls  i.prefecture_id i.province_id#year ,vce(cluster prefecture_id)  
margins, dydx(b)  //平均边际效应
outreg2 using  "Results\表1.doc",  keep(b ) se e(r2_p) bdec(3) rdec(3) nocons append


* 第4-6列：被解释变量=是否一窗办
reghdfe p2 b  $controls  ,vce(cluster prefecture_id) absorb(prefecture_id  province_id#year)   
outreg2 using  "Results\表1.doc",  keep(b ) se  bdec(3) rdec(3) nocons append


probit  p2 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
margins, dydx(b)  //平均边际效应
outreg2 using  "Results\表1.doc",  keep(b) se e(r2_p) bdec(3) rdec(3) nocons append


logit  p2 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
margins, dydx(b)  //平均边际效应
outreg2 using  "Results\表1.doc",  keep(b ) se e(r2_p) bdec(3) rdec(3) nocons append





*****************************************
**** 图4：平行趋势假设检验
*****************************************					  

* 1.生成开始受处理前后年份虚拟变量
gen event = year-year_treat       // 相对开始受处理年份的时间

replace event = -4 if event < -4  // 关注受处理前4期和后3期
replace event = 3  if event > 3


forvalues i=4(-1)1{               // 生成开始受处理前后年份虚拟变量
  gen pret`i'=(event==-`i'& treat==1 )
}

gen current=(event==0 & treat==1)

forvalues i=1(1)3{
  gen post`i'=(event==`i'& treat==1)
}

drop pret4                         // 以受处理前第4期为基期


* 2.将开始受处理前后年份虚拟变量引入模型，并画图
* 平行趋势画图：值得注意的是，开始受处理后2、3期虚拟变量与固定效应共线		
reghdfe p1 post* current  pret* $controls , vce(cluster prefecture_id) absorb(prefecture_id  province_id#year )  

coefplot, level(95) baselevels omitted ///
order(pret3 pret2 pret1 current post1) ///
keep(pret3 pret2 pret1 current post1) ///
vertical ///转置图形
coeflabels( pret3=-3 pret2=-2 pret1=-1 ///
current=0 post1=1 ) /// 
yline(0,lwidth(vthin) lpattern(solid) lcolor(teal)) ///
xline(4,lwidth(vthin) lpattern(solid) lcolor(teal)) ///
ytitle("是否跑一次") ///
xtitle("距离开始提升政务服务能力的年份") ///
msymbol(O) msize(small) mcolor(gs1) /// plot样式
addplot(line @b @at,lcolor(gs1) lwidth(medthick)) /// 增加点之间的连线
ciopts(lpattern(dash) recast(rcap) msize(medium)) /// CI为虚线上下封口
graphregion(color(white))  name(p1, replace)  // 白底	


reghdfe p2 post* current pret* $controls , vce(cluster prefecture_id) absorb(prefecture_id  province_id#year )  
  
coefplot, level(95) baselevels omitted ///
order(pret3 pret2 pret1 current post1) ///
keep(pret3 pret2 pret1 current post1) ///
vertical /// 转置图形
coeflabels( pret3=-3 pret2=-2 pret1=-1 ///
current=0 post1=1 ) /// 
yline(0,lwidth(vthin) lpattern(solid) lcolor(teal)) ///
xline(4,lwidth(vthin) lpattern(solid) lcolor(teal)) ///
ytitle("是否一窗办") ///
xtitle("距离开始提升政务服务能力的年份") ///
msymbol(O) msize(small) mcolor(gs1) /// plot样式
addplot(line @b @at,lcolor(gs1) lwidth(medthick)) /// 增加点之间的连线
ciopts(lpattern(dash) recast(rcap) msize(medium)) /// CI为虚线上下封口
graphregion(color(white))  name(p2, replace)  //白底	


graph combine p1 p2 , r(1) xsize(9) ysize(3) graphregion(color(white))
graph export "Results\图4.png", replace




*****************************************
**** 表2：企业成本效应的不同度量
*****************************************					  


* 第1列：被解释变量=几次办
reghdfe p3 b $controls ,vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
outreg2 using "Results\表2.doc",   keep(b ) se  bdec(3) rdec(3) nocons replace


* 第2列：被解释变量=几窗办
reghdfe p4 b $controls ,vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
outreg2 using "Results\表2.doc",  keep(b ) se  bdec(3) rdec(3) nocons append


* 第3列：被解释变量=是否省费用
reghdfe p5 b $controls ,vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
outreg2 using "Results\表2.doc",   keep(b ) se  bdec(3) rdec(3) nocons append


* 第4列：被解释变量=是否省时间
reghdfe p6 b $controls ,vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
outreg2 using "Results\表2.doc",  keep(b ) se  bdec(3) rdec(3) nocons append



		
*****************************************
**** 表3：IV回归分析
*****************************************										

* 第1-2列：被解释变量=是否跑一次
ivreghdfe p1 $controls (b = iv_b), absorb(prefecture_id province_id#year) cluster(prefecture_id) first savefirst  
// 输出第二阶段的结果
outreg2 using "Results\表3.doc", ctitle(Second stage) dec(3) se  keep(b) e(rkf) replace

// 输出第一阶段的结果
estimates restore _ivreg2_b
outreg2 first1 using "Results\表3.doc", ctitle(First stage) se dec(3) e(rkf) nocons keep(b iv_b) append


* 第3-4列：被解释变量=是否一窗办
ivreghdfe p2 $controls (b = iv_b), absorb(prefecture_id province_id#year) cluster(prefecture_id) first savefirst  
// 输出第二阶段的结果
outreg2 using "Results\表3.doc", ctitle(Second stage) dec(3) se  keep(b) e(rkf) append

// 输出第一阶段的结果
estimates restore _ivreg2_b
outreg2 first1 using "Results\表3.doc", ctitle(First stage) se dec(3) e(rkf)  nocons keep(b iv_b) append



















*------------------------------------------------------------------------------------------------------------*
*                                           附录图表
*------------------------------------------------------------------------------------------------------------*

*****************************************
**** 附录Ⅶ：表Ⅶ1
*****************************************						
use  "data.dta", clear

* 个体层面数据统计，手动将描述性统计结果加入附录Ⅶ的表Ⅶ1中
tabstat p1 p2 control_z1-control_z6 , statistics(mean N  sd)  by(treat) col(stat)  format(%10.2f)

ttable3 p1 p2 control_z1-control_z6 ,  by(treat) unequal format(%10.2f)  //注：ttable3输出的MeanDiff是控制组-处理组的差异，附录Ⅵ的表Ⅵ1中取相反数，得到处理组-控制组的差异


* 地市层面数据统计，手动将描述性统计结果加入附录Ⅶ的表Ⅶ1中
keep prefecture_id year treat control_x1-control_x4
duplicates drop

tabstat  control_x1-control_x4, statistics(mean N  sd)  by(treat) col(stat)  format(%10.2f)

ttable3  control_x1-control_x4,  by(treat) unequal  format(%10.2f)







*****************************************
**** 附录Ⅷ: 图Ⅷ1
*****************************************	
use "data.dta" , clear 
keep  prefecture_id 
duplicates drop

expand 7                                   // 根据处理组开始提及年份分布，生成随机抽取处理年份数据
bys prefecture_id: gen year=_n
replace year = year+2016

expand 9  if year ==2019|year==2023
expand 3  if year ==2017|year==2022
expand 5  if year ==2021

save "year_placebo.dta", replace

	
	
clear
set matsize 1000

foreach j of numlist 1/2 {    // 设置保留估计结果存储空间
	mat b`j' = J(500,1,0)
	mat se`j' = J(500,1,0)
	mat p`j' = J(500,1,0)
}	
	

set seed 1234

forvalues i=1/500{
	* 伪处理组的虚拟变量
	use "data.dta" , clear 
	keep prefecture_id
	qui duplicates drop     // 保留所有城市id
	qui sample 31, count    // 随机抽取31个处理组地市
	qui save "atchcity.dta",replace
	qui merge 1:m prefecture_id using "data.dta"
	gen groupnew=(_merge==3) // 生成伪处理组的虚拟变量
	qui save "matchcity`i'.dta", replace
		
	* 伪政策处理时间
	use "year_placebo.dta", clear 
	bsample 1, strata(prefecture_id)     //根据地市分组，每组随机抽取一个年份
	keep year
	qui save "matchyear.dta", replace
	mkmat year, matrix(sampleyear)
	

	use "matchcity`i'.dta", clear
	gen time = 0                 // 每个地市根据伪政策处理时间得到时间层面虚拟变量
	foreach j of numlist 1/45 {
		qui replace time = 1 if (prefecture_id == `j' & year >= sampleyear[`j',1])
	}	


	gen  did=time*groupnew    //did=伪处理组的虚拟变量×伪处理时间虚拟变量
	
	
	* 是否跑一次
	qui reghdfe p1 did $controls , vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
	
	mat b1[`i',1] = _b[did]          //存储估计结果
	mat se1[`i',1] = _se[did]
	scalar df_r = e(df_r)
	mat p1[`i',1] = 2*ttail(df_r,abs(_b[did]/_se[did]))
	
	
	* 是否一窗办
	qui reghdfe p2 did $controls , vce(cluster prefecture_id) absorb( prefecture_id province_id#year)   
	
	mat b2[`i',1] = _b[did]          //存储估计结果
	mat se2[`i',1] = _se[did]
	scalar df_r = e(df_r)
	mat p2[`i',1] = 2*ttail(df_r,abs(_b[did]/_se[did]))
	
	
	}
	
	
	
foreach j of numlist 1/2 {
	svmat b`j', names(coef`j')
	svmat se`j', names(se`j')
	svmat p`j', names(pvalue`j')

	drop if pvalue`j' == .
	label var pvalue`j' p值
	label var coef`j' 估计系数
}	


save "placebo.dta", replace  // 保存数据


* 画图
graph set window fontface "Times New Roman"
graph set window fontfacesans "宋体"  // 设置图形输出的字体

	
twoway (scatter pvalue1 coef1,  ///
    yline(0.1,lp(shortdash)) xline(0.179,lp(shortdash))                     ///
    xtitle("{stSans:估计系数}") ytitle("{stSans:P}" "{stSans:值}",           ///
    orientation(h)) msymbol(smcircle_hollow) mcolor(grey) legend(off))       ///
    (kdensity coef1, yaxis(2) title("{stSans:是否跑一次}") name(placebo1, replace) graphregion(color(white))) 
	
	
twoway (scatter pvalue2 coef2,  ///
    yline(0.1,lp(shortdash)) xline(0.216,lp(shortdash))                     ///
    xtitle("{stSans:估计系数}") ytitle("{stSans:P}" "{stSans:值}",           ///
    orientation(h)) msymbol(smcircle_hollow) mcolor(grey) legend(off))       ///
    (kdensity coef2, yaxis(2) title("{stSans:是否一窗办}") name(placebo2, replace) graphregion(color(white)))
	
graph combine  placebo1 placebo2, r(1)	xsize(9) ysize(3) graphregion(color(white))
graph export "Results\图Ⅷ1.png", replace


* 删除临时文件
erase "atchcity.dta"
erase "matchyear.dta"
erase "year_placebo.dta"
erase "placebo.dta"

forvalue i=1/500{
    erase  "matchcity`i'.dta"
}





					  
*****************************************
**** 附录Ⅷ:表Ⅷ1
*****************************************	
* 第1-2列：截面匹配
use  "data.dta", clear

psmatch2 treat $controls , outcome(p1) logit neighbor(2) ties common ate caliper(0.05)
reghdfe p1 b  $controls if _support == 1, vce(cluster prefecture_id) absorb(prefecture_id province_id#year) 
outreg2 using "Results\表Ⅷ1.doc",  keep(b) se  bdec(3) rdec(3) nocons replace



psmatch2 treat $controls , outcome(p2) logit neighbor(2) ties common ate caliper(0.05)
reghdfe p2 b  $controls   if _support == 1, vce(cluster prefecture_id) absorb(prefecture_id province_id#year) 
outreg2 using "Results\表Ⅷ1.doc",  keep(b) se  bdec(3) rdec(3) nocons append



					  
					  
* 第3-4列：逐年匹配
* 是否跑一次
use  "data.dta", clear
forvalue i = 2018/2023{                             // 逐年进行匹配
	  preserve
		  capture {
			  
			  keep if year == `i'
			  set seed 0000
			  gen  norvar_2 = rnormal()
			  sort norvar_2
			  psmatch2 treat $controls , outcome(p1) logit neighbor(2)  ties common ate caliper(0.05)
			  save "`i'.dta", replace
			  }
	  restore
	  }

		  
clear all
use "2018.dta", clear                                // 将匹配后的各年合并到一起
forvalue k =2019/2023 {
	capture {
		  append using "`k'.dta"
		  }
	  }
		  
reghdfe p1 b $controls if _support == 1, vce(cluster prefecture_id) absorb(prefecture_id province_id#year )   // did估计
outreg2 using "Results\表Ⅷ1.doc", keep(b) se  bdec(3) rdec(3) nocons append	

					  
* 删除临时文件	
forvalue k =2018/2023 {
	capture {
		  erase "`k'.dta"
		  }
		  
	  }
		
		
		
		
* 是否一窗办
use  "data.dta", clear
forvalue i = 2018/2023{                             // 逐年进行匹配
	  preserve
		  capture {
			  
			  keep if year == `i'
			  set seed 0000
			  gen  norvar_2 = rnormal()
			  sort norvar_2
			  psmatch2 treat $controls , outcome(p2) logit neighbor(2)  ties common ate caliper(0.05)
			  save "`i'.dta", replace
			  }
	  restore
	  }

		  
clear all
use "2018.dta", clear                                // 将匹配后的各年合并到一起
forvalue k =2019/2023 {
	capture {
		  append using "`k'.dta"
		  }
	  }
		  
reghdfe p2 b $controls if _support == 1, vce(cluster prefecture_id) absorb(prefecture_id province_id#year )   // did估计
outreg2 using "Results\表Ⅷ1.doc", keep(b) se  bdec(3) rdec(3) nocons append	

					  
* 删除临时文件	
forvalue k =2018/2023 {
	capture {
		  erase "`k'.dta"
		  }
		  
	  }		
		
		
		
		
		
		
		
					  
*****************************************
**** 附录Ⅷ:表Ⅷ2
*****************************************	
use  "data.dta", clear  

* 第1-2列：Sun和Abraham (2021)
gen utreat = 1-treat              //生成控制组虚拟变量

gen k = year - year_treat         //生成开始受处理前后年份虚拟变量

replace k = -4 if k < -4          //只关注前4期和后3期
replace k = 3 if k > 3 & k!=.

forvalues l = 0/3 {
  gen g`l' = k==`l'
  }
forvalues l = 1/4 {
  gen g_`l' = k==-`l'
  }
  
drop g_4                           //以开始受处理前第4期为基准组
  

eventstudyinteract p1  g0-g3 g_*, vce(cluster prefecture_id) absorb(prefecture_id province_id#year) cohort(year_treat) control_cohort(utreat)    covariates($controls )      //后2期和后3期共线
matrix b = e(b_iw)
matrix V = e(V_iw)
ereturn post b V
lincom (g0 + g1 )/2       //计算平均处理效应，手动加入表Ⅷ2


eventstudyinteract p2  g0-g3 g_*, vce(cluster prefecture_id) absorb(prefecture_id province_id#year) cohort(year_treat) control_cohort(utreat)    covariates($controls )      //后2期和后3期共线
matrix b = e(b_iw)
matrix V = e(V_iw)
ereturn post b V
lincom (g0 + g1 )/2		//计算平均处理效应，手动加入表Ⅷ2
 
 
 
 
* 第3-4列：Borusyak et al.（2024）
did_imputation p1 prefecture_id year year_treat,fe(prefecture_id province_id#year) /*
    */ controls( $controls ) tol(1000) maxit(10000) autosample cluster(prefecture_id)    //计算平均处理效应，将变量tau的系数估计手动加入表Ⅷ2
 
did_imputation p2 prefecture_id year year_treat,fe(prefecture_id province_id#year) /*
    */ controls( $controls ) tol(1000) maxit(10000) autosample cluster(prefecture_id)    //计算平均处理效应，将变量tau的系数估计手动加入表Ⅷ2







*****************************************
**** 附录Ⅷ:表Ⅷ3 
*****************************************
/*
注："data_稳健.dta"包含的数据为:所有不先于省提及改革的处理组地市和控制组地市，所有年份的企业样本
    未提供"data_稳健.dta"数据，具体回归结果见“结果截图.docx”	
*/
* 第1-2列：改变开始处理年份
use  "data_稳健.dta", clear 
drop if treat==1 & treat_continuous==0       // 去除处理组中不连续提及地市样本
drop if b_f1==.                              // 去除终止提及之后的年份样本	
* b_f1：地市当年是否提升政务服务能力（准自然实验开始年份提前一年）

reghdfe p1 b_f1 $controls , vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
outreg2 using "Results\表Ⅷ3.doc",  keep(b_f1 ) se  bdec(3) rdec(3) nocons replace


reghdfe p2 b_f1 $controls , vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
outreg2 using "Results\表Ⅷ3.doc",  keep(b_f1 ) se  bdec(3) rdec(3) nocons append




* 第3-4列：改变处理组
use  "data_稳健.dta", clear
drop if b==.                              // 去除终止提及之后的年份样本

reghdfe p1 b $controls , vce(cluster prefecture_id) absorb(prefecture_id province_id#year)   
outreg2 using "Results\表Ⅷ3.doc",  keep(b ) se  bdec(3) rdec(3) nocons append


reghdfe p2 b $controls , vce(cluster prefecture_id) absorb(prefecture_id  province_id#year)   
outreg2 using "Results\表Ⅷ3.doc",  keep(b ) se  bdec(3) rdec(3) nocons append



				  
* 第5-6列：持续性影响
use  "data_稳健.dta", clear
drop if treat==1 & treat_continuous==0            // 去除处理组中不连续提及地市样本
drop if b_suspend==.                              // 剔除处理组地市提及改革的年份样本

* b_suspend：地市当年是否终止提及改革

reg p1 b_suspend $controls i.prefecture_id i.province_id#year,vce(cluster prefecture_id)
outreg2 using "Results\表Ⅷ3.doc",  keep(b_suspend) se  bdec(3) rdec(3) nocons append

reg p2 b_suspend $controls i.prefecture_id i.province_id#year,vce(cluster prefecture_id)
outreg2 using "Results\表Ⅷ3.doc",  keep(b_suspend ) se  bdec(3) rdec(3) nocons append					  



*****************************************
**** 附录Ⅸ:表A1
*****************************************	

use  "data.dta", clear

* 第1-2列：poisson
poisson  p3 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
outreg2 using "Results\表A1.doc",  keep(b ) se e(r2_p) bdec(3) rdec(3) nocons replace

poisson  p4 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
outreg2 using "Results\表A1.doc",  keep(b ) se e(r2_p) bdec(3) rdec(3) nocons append


* 第3-4列：probit
probit  p5 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
outreg2 using  "Results\表A1.doc",  keep(b) se e(r2_p) bdec(3) rdec(3) nocons append


probit  p6 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
outreg2 using  "Results\表A1.doc",  keep(b) se e(r2_p) bdec(3) rdec(3) nocons append


* 第5-6列：logot
logit  p5 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
outreg2 using  "Results\表A1.doc",  keep(b ) se e(r2_p) bdec(3) rdec(3) nocons append

logit  p6 b  $controls i.prefecture_id i.province_id#year ,vce(cluster prefecture_id) 
outreg2 using  "Results\表A1.doc",  keep(b ) se e(r2_p) bdec(3) rdec(3) nocons append


clear 
log close